/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.droids.wicket.component;

import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;
import java.util.Iterator;
import java.util.Queue;

import org.apache.droids.api.Droid;
import org.apache.droids.api.Task;
import org.apache.droids.impl.MultiThreadedTaskMaster;
import org.apache.droids.monitor.SimpleWorkMonitor;
import org.apache.droids.monitor.WorkBean;
import org.apache.wicket.ajax.AbstractAjaxTimerBehavior;
import org.apache.wicket.ajax.AjaxRequestTarget;
import org.apache.wicket.extensions.ajax.markup.html.AjaxEditableChoiceLabel;
import org.apache.wicket.markup.html.WebMarkupContainer;
import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.panel.Panel;
import org.apache.wicket.markup.repeater.Item;
import org.apache.wicket.markup.repeater.RefreshingView;
import org.apache.wicket.markup.repeater.data.DataView;
import org.apache.wicket.model.AbstractReadOnlyModel;
import org.apache.wicket.model.IModel;
import org.apache.wicket.model.PropertyModel;
import org.apache.wicket.spring.injection.annot.SpringBean;
import org.apache.wicket.util.lang.Bytes;
import org.apache.wicket.util.time.Duration;

public class DroidPanel extends Panel
{
  static final SimpleDateFormat timeFormat = new SimpleDateFormat( "HH:mm:ss" );

  @SpringBean( name="droid" )
  Droid droid;
  
  @SpringBean( name="taskQueue" )
  Queue taskQueue;
  
  @SpringBean( name="taskMaster" )
  MultiThreadedTaskMaster taskMaster;
  
  @SpringBean( name="runningMonitor" )
  SimpleWorkMonitor monitor;

  
  public DroidPanel(String id) {
    super(id);

    this.setOutputMarkupId( true );
    this.setVersioned( false );
    
    final WebMarkupContainer info = new WebMarkupContainer( "info" );
    info.setOutputMarkupId( true );
    
    // SHOW INFO
    info.add( new Label( "currentTime", new AbstractReadOnlyModel<String>() {
      @Override
      public String getObject() {
        return timeFormat.format( new Date() );
      }
    }));

    info.add( new Label( "startedTime", new AbstractReadOnlyModel<String>() {
      @Override
      public String getObject() {
        Date d = droid.getTaskMaster().getStartTime();
        return (d==null)?"" : timeFormat.format( d );
      }
    }));

    info.add( new Label( "finishedTime", new AbstractReadOnlyModel<String>() {
      @Override
      public String getObject() {
        Date d = droid.getTaskMaster().getFinishedWorking();
        return (d==null)?"" : timeFormat.format( d );
      }
    }));

    info.add( new Label( "completedTasks", new AbstractReadOnlyModel<Long>() {
      @Override
      public Long getObject() {
        return droid.getTaskMaster().getCompletedTasks();
      }
    }));
    
    info.add( new Label( "lastTask", new AbstractReadOnlyModel<String>() {
      @Override
      public String getObject() {
        Task task = droid.getTaskMaster().getLastCompletedTask();
        return (task==null)?"" : task.getId();
      }
    }));

    info.add( new Label( "memory", new AbstractReadOnlyModel<CharSequence>() {
      @Override
      public CharSequence getObject() {
        Runtime runtime = Runtime.getRuntime();
        Bytes total = Bytes.bytes( runtime.totalMemory() );
        Bytes free = Bytes.bytes( runtime.freeMemory() ); 
        Bytes max = Bytes.bytes( runtime.maxMemory() );
        Bytes used = Bytes.bytes( total.bytes()-free.bytes() );
        int per = (int)((used.bytes()/(float)max.bytes())*100.0f);
        
        return used + "("+total+")/" +max + " (%"+per+")";
      }
    }));
    add( info );
    
    WebMarkupContainer classinfo = new WebMarkupContainer( "classinfo" );
    classinfo.add( new Label( "droid", droid.toString() ) );
    classinfo.add( new Label( "queue", taskQueue.toString() ) );
    classinfo.add( new Label( "taskMaster", taskMaster.toString() ) );
    add( classinfo );
    
    // TASK MASTER
    WebMarkupContainer tmDIV = new WebMarkupContainer( "taskMaster" );
    final WebMarkupContainer workers = new WebMarkupContainer( "workers" );
    workers.setOutputMarkupId( true );
    workers.add(new DataView<WorkBean>("task", new WorkBeanDataProvider( 
        new AbstractReadOnlyModel<SimpleWorkMonitor>() {
          @Override
          public SimpleWorkMonitor getObject() {
            return monitor;
          }
    } ) )
    {
      @Override
      protected void populateItem(final Item<WorkBean> item)
      {
        WorkBean bean = item.getModelObject();
        long elapsed = System.currentTimeMillis() - bean.getStartTime().getTime();
        item.add( new Label("time", elapsed+"" )); //DurationFormatUtil.formatDurationHMS(elapsed) ) );
        item.add( new Label("text", bean.getTask().toString() ) );
      }
    });
    tmDIV.add( workers );
    WebMarkupContainer threads = new WebMarkupContainer( "threads" );
    threads.add( new AjaxEditableChoiceLabel<Integer>("count", 
        new PropertyModel<Integer>( taskMaster, "maxThreads" )
    , Arrays.asList( 1,2,3,4,5,6,7,8 ) ) );  
    tmDIV.add( threads );
    add( tmDIV );
    
    
    
    // QUEUE
    final WebMarkupContainer queueDIV = new WebMarkupContainer( "queue" );
    queueDIV.add( new RefreshingView<Task>("task")
    {
      /**
       * Return an iterator over models for items in the view
       */
      @Override
      protected Iterator<IModel<Task>> getItemModels()
      {
        return taskQueue.iterator();
      }

      @Override
      protected void populateItem(final Item<Task> item) {
        Task task = item.getModelObject();
        long elapsed = System.currentTimeMillis() - task.getTaskDate().getTime();
        item.add( new Label("time", elapsed+"" )); //DurationFormatUtil.formatDurationHMS(elapsed) ) );
        item.add( new Label("text", task.toString() ) );
      }
    });
    queueDIV.add( new Label( "more", new AbstractReadOnlyModel<String>() {
      @Override
      public String getObject() {
        return taskQueue.size() + " tasks.";
      }
    }));
    queueDIV.setOutputMarkupId( true );
    add( queueDIV );
    
    
    
    // Refresh the view every second...
    add(new AbstractAjaxTimerBehavior(Duration.seconds(1))
    {
      @Override
      protected void onTimer(AjaxRequestTarget target)
      {
        target.addComponent( info );
        target.addComponent( queueDIV );
        target.addComponent( workers );
      }
    });
  }
}
